@@ -11,21 +11,23 @@
 import math
 import sys
 from contextlib import contextmanager
+from typing import Optional
 
 from hypothesis import HealthCheck, Phase, assume, settings, strategies as st
 from hypothesis.control import current_build_context
 from hypothesis.errors import InvalidArgument
 from hypothesis.internal.conjecture import engine as engine_module
+from hypothesis.internal.conjecture.choice import ChoiceT
 from hypothesis.internal.conjecture.data import (
     COLLECTION_DEFAULT_MAX_SIZE,
     ConjectureData,
     IRNode,
-    IRType,
     Status,
 )
 from hypothesis.internal.conjecture.engine import BUFFER_SIZE, ConjectureRunner
 from hypothesis.internal.conjecture.utils import calc_label_from_name
 from hypothesis.internal.entropy import deterministic_PRNG
+from hypothesis.internal.escalation import InterestingOrigin
 from hypothesis.internal.floats import SMALLEST_SUBNORMAL, sign_aware_lte
 from hypothesis.internal.intervalsets import IntervalSet
 from hypothesis.strategies._internal.strings import OneCharStringStrategy, TextStrategy
@@ -40,6 +42,22 @@ TEST_SETTINGS = settings(
 )
 
 
+def interesting_origin(n: Optional[int] = None) -> InterestingOrigin:
+    """
+    Creates and returns an InterestingOrigin, parameterized by n, such that
+    interesting_origin(n) == interesting_origin(m) iff n = m.
+
+    Since n=None may by chance concide with an explicitly-passed value of n, I
+    recommend not mixing interesting_origin() and interesting_origin(n) in the
+    same test.
+    """
+    try:
+        int("not an int")
+    except Exception as e:
+        origin = InterestingOrigin.from_exception(e)
+        return origin._replace(lineno=n if n is not None else origin.lineno)
+
+
 def run_to_data(f):
     with deterministic_PRNG():
         runner = ConjectureRunner(f, settings=TEST_SETTINGS)
@@ -122,6 +140,28 @@ def clamped_shrink_towards(kwargs):
     return v
 
 
+@st.composite
+def integer_weights(draw, min_value=None, max_value=None):
+    # Sampler doesn't play well with super small floats, so exclude them
+    weights = draw(
+        st.dictionaries(
+            st.integers(min_value=min_value, max_value=max_value),
+            st.floats(0.001, 1),
+            min_size=1,
+            max_size=255,
+        )
+    )
+    # invalid to have a weighting that disallows all possibilities
+    assume(sum(weights.values()) != 0)
+    # re-normalize probabilities to sum to some arbitrary target < 1
+    target = draw(st.floats(0.001, 0.999))
+    factor = target / sum(weights.values())
+    weights = {k: v * factor for k, v in weights.items()}
+    # float rounding error can cause this to fail.
+    assume(0.001 <= sum(weights.values()) <= 0.999)
+    return weights
+
+
 @st.composite
 def integer_kwargs(
     draw,
@@ -164,22 +204,7 @@ def integer_kwargs(
         min_val = max(min_value, forced) if forced is not None else min_value
         max_value = draw(st.integers(min_value=min_val))
 
-        # Sampler doesn't play well with super small floats, so exclude them
-        weights = draw(
-            st.dictionaries(
-                st.integers(min_value=min_value, max_value=max_value),
-                st.floats(0.001, 1),
-                max_size=255,
-            )
-        )
-        # invalid to have a weighting that disallows all possibilities
-        assume(sum(weights.values()) != 0)
-        # re-normalize probabilities to sum to some arbitrary target < 1
-        target = draw(st.floats(0.001, 0.999))
-        factor = target / sum(weights.values())
-        weights = {k: v * factor for k, v in weights.items()}
-        # float rounding error can cause this to fail.
-        assume(sum(weights.values()) == target)
+        weights = draw(integer_weights(min_value, max_value))
     else:
         if use_min_value:
             min_value = draw(st.integers(max_value=forced))
@@ -356,10 +381,11 @@ def draw_value(ir_type, kwargs):
 
 
 @st.composite
-def ir_nodes(draw, *, was_forced=None, ir_type=None):
-    if ir_type is None:
+def ir_nodes(draw, *, was_forced=None, ir_types=None):
+    if ir_types is None:
         (ir_type, kwargs) = draw(ir_types_and_kwargs())
     else:
+        ir_type = draw(st.sampled_from(ir_types))
         kwargs = draw(kwargs_strategy(ir_type))
     # ir nodes don't include forced in their kwargs. see was_forced attribute
     del kwargs["forced"]
@@ -369,7 +395,7 @@ def ir_nodes(draw, *, was_forced=None, ir_type=None):
     return IRNode(ir_type=ir_type, value=value, kwargs=kwargs, was_forced=was_forced)
 
 
-def ir(*values: list[IRType]) -> list[IRNode]:
+def ir(*values: list[ChoiceT]) -> list[IRNode]:
     """
     For inline-creating an ir node or list of ir nodes, where you don't care about the
     kwargs. This uses maximally-permissable kwargs and infers the ir_type you meant
@@ -428,3 +454,35 @@ def ir(*values: list[IRType]) -> list[IRNode]:
         )
 
     return tuple(nodes)
+
+
+def float_kw(
+    min_value=-math.inf,
+    max_value=math.inf,
+    *,
+    allow_nan=True,
+    smallest_nonzero_magnitude=SMALLEST_SUBNORMAL,
+):
+    return {
+        "min_value": min_value,
+        "max_value": max_value,
+        "allow_nan": allow_nan,
+        "smallest_nonzero_magnitude": smallest_nonzero_magnitude,
+    }
+
+
+def integer_kw(min_value=None, max_value=None, *, weights=None, shrink_towards=0):
+    return {
+        "min_value": min_value,
+        "max_value": max_value,
+        "weights": weights,
+        "shrink_towards": shrink_towards,
+    }
+
+
+def string_kw(intervals, *, min_size=0, max_size=COLLECTION_DEFAULT_MAX_SIZE):
+    return {"intervals": intervals, "min_size": min_size, "max_size": max_size}
+
+
+# we could in theory define bytes_kw and boolean_kw, but without any
+# default kw values they aren't really a time save.
