@@ -20,12 +20,12 @@ from hypothesis.internal.conjecture.junkdrawer import (
     NotFound,
     SelfOrganisingList,
     binary_search,
-    clamp,
     endswith,
     replace_all,
     stack_depth_of_caller,
     startswith,
 )
+from hypothesis.internal.floats import clamp, float_to_int, sign_aware_lte
 
 
 def test_out_of_range():
@@ -62,24 +62,35 @@ def test_pop():
         x.pop()
 
 
-@example(1, 5, 10)
-@example(1, 10, 5)
-@example(5, 10, 5)
-@example(5, 1, 10)
-@given(st.integers(), st.integers(), st.integers())
-def test_clamp(lower, value, upper):
-    lower, upper = sorted((lower, upper))
+@st.composite
+def clamp_inputs(draw):
+    lower = draw(st.floats(allow_nan=False))
+    value = draw(st.floats(allow_nan=False))
+    upper = draw(st.floats(min_value=lower, allow_nan=False))
+    return (lower, value, upper)
 
+
+@example((1, 5, 10))
+@example((1, 10, 5))
+@example((5, 10, 5))
+@example((5, 1, 10))
+@example((-5, 0.0, -0.0))
+@example((0.0, -0.0, 5))
+@example((-0.0, 0.0, 0.0))
+@example((-0.0, -0.0, 0.0))
+@given(clamp_inputs())
+def test_clamp(input):
+    lower, value, upper = input
     clamped = clamp(lower, value, upper)
 
-    assert lower <= clamped <= upper
-
-    if lower <= value <= upper:
-        assert value == clamped
+    assert sign_aware_lte(lower, clamped)
+    assert sign_aware_lte(clamped, upper)
+    if sign_aware_lte(lower, value) and sign_aware_lte(value, upper):
+        assert float_to_int(value) == float_to_int(clamped)
     if lower > value:
-        assert clamped == lower
+        assert float_to_int(clamped) == float_to_int(lower)
     if value > upper:
-        assert clamped == upper
+        assert float_to_int(clamped) == float_to_int(upper)
 
 
 # this would be more robust as a stateful test, where each rule is a list operation
