@@ -8,16 +8,17 @@
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
 
+import itertools
+
 import pytest
 
 from hypothesis import HealthCheck, Phase, settings, strategies as st
-from hypothesis.database import InMemoryExampleDatabase
-from hypothesis.internal.compat import int_to_bytes
+from hypothesis.database import InMemoryExampleDatabase, ir_to_bytes
 from hypothesis.internal.conjecture.data import Status
 from hypothesis.internal.conjecture.engine import ConjectureRunner, RunIsComplete
 from hypothesis.internal.entropy import deterministic_PRNG
 
-from tests.conjecture.common import ir
+from tests.conjecture.common import interesting_origin
 
 
 def test_pareto_front_contains_different_interesting_reasons():
@@ -25,7 +26,8 @@ def test_pareto_front_contains_different_interesting_reasons():
 
         def test(data):
             data.target_observations[""] = 1
-            data.mark_interesting(data.draw_integer(0, 2**4 - 1))
+            n = data.draw_integer(0, 2**4 - 1)
+            data.mark_interesting(interesting_origin(n))
 
         runner = ConjectureRunner(
             test,
@@ -132,7 +134,7 @@ def test_clears_defunct_pareto_front():
         )
 
         for i in range(256):
-            db.save(runner.pareto_key, bytes([i, 0]))
+            db.save(runner.pareto_key, ir_to_bytes((i, 0)))
 
         runner.run()
 
@@ -159,8 +161,8 @@ def test_down_samples_the_pareto_front():
             database_key=b"stuff",
         )
 
-        for i in range(10000):
-            db.save(runner.pareto_key, int_to_bytes(i, 2))
+        for n1, n2 in itertools.product(range(256), range(256)):
+            db.save(runner.pareto_key, ir_to_bytes((n1, n2)))
 
         with pytest.raises(RunIsComplete):
             runner.reuse_existing_examples()
@@ -172,8 +174,8 @@ def test_stops_loading_pareto_front_if_interesting():
     with deterministic_PRNG():
 
         def test(data):
-            data.draw_integer(0, 2**8 - 1)
-            data.draw_integer(0, 2**8 - 1)
+            data.draw_integer()
+            data.draw_integer()
             data.mark_interesting()
 
         db = InMemoryExampleDatabase()
@@ -189,8 +191,8 @@ def test_stops_loading_pareto_front_if_interesting():
             database_key=b"stuff",
         )
 
-        for i in range(10000):
-            db.save(runner.pareto_key, int_to_bytes(i, 2))
+        for n1, n2 in itertools.product(range(256), range(256)):
+            db.save(runner.pareto_key, ir_to_bytes((n1, n2)))
 
         runner.reuse_existing_examples()
 
@@ -231,12 +233,12 @@ def test_optimises_the_pareto_front():
         settings=settings(max_examples=10000, database=InMemoryExampleDatabase()),
         database_key=b"stuff",
     )
-    runner.cached_test_function_ir(ir(255) * 20 + ir(0))
+    runner.cached_test_function_ir([255] * 20 + [0])
     runner.pareto_optimise()
 
     assert len(runner.pareto_front) == 6
     for i, data in enumerate(runner.pareto_front):
-        assert list(data.buffer) == [1] * i + [0]
+        assert data.choices == (1,) * i + (0,)
 
 
 def test_does_not_optimise_the_pareto_front_if_interesting():
@@ -252,7 +254,7 @@ def test_does_not_optimise_the_pareto_front_if_interesting():
         database_key=b"stuff",
     )
 
-    runner.cached_test_function_ir(ir(0))
+    runner.cached_test_function_ir([0])
     runner.pareto_optimise = None
     runner.optimise_targets()
 
@@ -274,7 +276,7 @@ def test_stops_optimising_once_interesting():
         database_key=b"stuff",
     )
 
-    data = runner.cached_test_function_ir(ir(hi))
+    data = runner.cached_test_function_ir([hi])
     assert data.status == Status.VALID
     runner.pareto_optimise()
     assert runner.call_count <= 20
