@@ -35,7 +35,7 @@ from hypothesis.database import (
 )
 from hypothesis.errors import HypothesisWarning
 from hypothesis.internal.compat import WINDOWS
-from hypothesis.internal.conjecture.data import ir_value_equal
+from hypothesis.internal.conjecture.choice import choice_equal
 from hypothesis.stateful import Bundle, RuleBasedStateMachine, rule
 from hypothesis.strategies import binary, lists, tuples
 from hypothesis.utils.conventions import not_set
@@ -478,17 +478,14 @@ def test_background_write_database():
 @example(ir("a"))
 @example(ir(b"a"))
 @example(ir(b"a" * 50))
-def test_ir_nodes_rountrips(nodes1):
+def test_ir_nodes_roundtrips(nodes1):
     s1 = ir_to_bytes([n.value for n in nodes1])
     assert isinstance(s1, bytes)
     ir2 = ir_from_bytes(s1)
     assert len(nodes1) == len(ir2)
 
     for n1, v2 in zip(nodes1, ir2):
-        v1 = n1.value
-        ir_type = n1.ir_type
-        assert type(v1) is type(v2)
-        assert ir_value_equal(ir_type, v1, v2)
+        assert choice_equal(n1.value, v2)
 
     s2 = ir_to_bytes(ir2)
     assert s1 == s2
