@@ -8,7 +8,6 @@
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
 
-import base64
 import re
 from collections import defaultdict
 from typing import ClassVar
@@ -27,6 +26,7 @@ from hypothesis import (
     strategies as st,
 )
 from hypothesis.control import current_build_context
+from hypothesis.core import encode_failure
 from hypothesis.database import ExampleDatabase
 from hypothesis.errors import DidNotReproduce, Flaky, InvalidArgument, InvalidDefinition
 from hypothesis.internal.entropy import deterministic_PRNG
@@ -1074,7 +1074,7 @@ def test_uses_seed(capsys):
 
 
 def test_reproduce_failure_works():
-    @reproduce_failure(__version__, base64.b64encode(b"\x00\x00\x01\x00\x00\x00"))
+    @reproduce_failure(__version__, encode_failure([0, 1]))
     class TrivialMachine(RuleBasedStateMachine):
         @rule()
         def oops(self):
@@ -1085,7 +1085,7 @@ def test_reproduce_failure_works():
 
 
 def test_reproduce_failure_fails_if_no_error():
-    @reproduce_failure(__version__, base64.b64encode(b"\x00\x00\x01\x00\x00\x00"))
+    @reproduce_failure(__version__, encode_failure([0, 1]))
     class TrivialMachine(RuleBasedStateMachine):
         @rule()
         def ok(self):
