@@ -17,7 +17,6 @@ from hypothesis.internal.conjecture.engine import ConjectureData, ConjectureRunn
 from hypothesis.strategies._internal import SearchStrategy
 
 POISON = "POISON"
-
 MAX_INT = 2**32 - 1
 
 
@@ -49,13 +48,10 @@ class PoisonedTree(SearchStrategy):
                 return (None,)
 
 
-LOTS = 10**6
-
-
 TEST_SETTINGS = settings(
     database=None,
     suppress_health_check=list(HealthCheck),
-    max_examples=LOTS,
+    max_examples=10**6,
     deadline=None,
 )
 
@@ -85,19 +81,23 @@ def test_can_reduce_poison_from_any_subtree(size, seed):
     runner.generate_new_examples()
     runner.shrink_interesting_examples()
     (data,) = runner.interesting_examples.values()
+    assert len(ConjectureData.for_choices(data.choices).draw(strat)) == size
 
-    assert len(ConjectureData.for_buffer(data.buffer).draw(strat)) == size
-
-    starts = [b.start for b in data.blocks if b.length == 2]
-    assert len(starts) % 2 == 0
+    # find the nodes corresponding to n1 and n2
+    nodes = [
+        node
+        for node in data.ir_nodes
+        if node.ir_type == "integer" and node.kwargs["max_value"] == 2**16 - 1
+    ]
+    assert len(nodes) % 2 == 0
 
     marker = bytes([1, 2, 3, 4])
-    for i in range(0, len(starts), 2):
+    for i in range(0, len(nodes), 2):
         # Now for each leaf position in the tree we try inserting a poison
         # value artificially. Additionally, we add a marker to the end that
         # must be preserved. The marker means that we are not allow to rely on
-        # discarding the end of the buffer to get the desired shrink.
-        u = starts[i]
+        # discarding the end of the choice sequence to get the desired shrink.
+        node = nodes[i]
 
         def test_function_with_poison(data):
             v = data.draw(strat)
@@ -108,11 +108,15 @@ def test_can_reduce_poison_from_any_subtree(size, seed):
         runner = ConjectureRunner(
             test_function_with_poison, random=random, settings=TEST_SETTINGS
         )
-        runner.cached_test_function(
-            data.buffer[:u] + bytes([255]) * 4 + data.buffer[u + 4 :] + marker
+        # replace n1 and n2 with 2**16 - 1 to insert a poison value here
+        runner.cached_test_function_ir(
+            data.choices[: node.index]
+            + (2**16 - 1, 2**16 - 1)
+            + (data.choices[node.index + 2 :])
+            + (marker,)
         )
         assert runner.interesting_examples
 
         runner.shrink_interesting_examples()
         (shrunk,) = runner.interesting_examples.values()
-        assert ConjectureData.for_buffer(shrunk.buffer).draw(strat) == (POISON,)
+        assert ConjectureData.for_choices(shrunk.choices).draw(strat) == (POISON,)
